@@ -21,6 +21,7 @@ import pygobject
 import sendSMS
 import locale
 import dbus
+import pango
 from portrait import FremantleRotation
 
 # By using apt-cache search, the following packages are required:
@@ -100,8 +101,8 @@ class StrokeOrder():
 	def queryString(self, stroke):
 		# Takes in the inputted stroke sequence and display the candidate characters for selection
 		if stroke!="":
-			if (len(self.strokesCodeInputted)<=4 and "6" not in self.strokesCodeInputted and "7" not in self.strokesCodeInputted) or ("%" in self.strokesCodeInputted):	# For <=4 strokes, since there would be a huge amount of characters having strokes beginning with the same stroke sequence, limit the number of candidate characters to 50 for the sake of performance
-				self.cur.execute("SELECT DISTINCT Character FROM StrokeOrder WHERE Strokes LIKE '" + stroke + "%' ORDER BY Frequency DESC, Strokes ASC, Character ASC LIMIT 50")
+			if (len(self.strokesCodeInputted)<=4 and "6" not in self.strokesCodeInputted and "7" not in self.strokesCodeInputted) or ("%" in self.strokesCodeInputted):	# For <=4 strokes, since there would be a huge amount of characters having strokes beginning with the same stroke sequence, limit the number of candidate characters to 75 for the sake of performance
+				self.cur.execute("SELECT DISTINCT Character FROM StrokeOrder WHERE Strokes LIKE '" + stroke + "%' ORDER BY Frequency DESC, Strokes ASC, Character ASC LIMIT 75")
 			else:
 				self.cur.execute("SELECT DISTINCT Character FROM StrokeOrder WHERE Strokes LIKE '" + stroke + "%' ORDER BY Frequency DESC, Strokes ASC, Character ASC")	# For >4 strokes, display all candidate characters
 			self.con.commit()
@@ -111,7 +112,20 @@ class StrokeOrder():
 
 		if stroke!="":
 			for row in self.cur.fetchall():
-					self.candidateChars.append_text(row[0])	# Generate candidate character selection list
+				self.candidateChars.append_text(row[0])	# Generate candidate character selection list
+
+	def queryStrokes(self, queryStr):
+		# Takes in a Chinese character and returns its stroke orders
+		querychar = queryStr[0:1]
+		print querychar
+		sQuery = "SELECT DISTINCT Strokes FROM StrokeOrder WHERE Character='" + querychar + "' LIMIT 1"
+		print sQuery
+		self.cur.execute(sQuery)
+		self.con.commit()
+		result=""
+		for row in self.cur.fetchall():
+			result = row[0]
+		return result
 	
 	def queryAssoc(self, charKey):		
 		# Takes in a Chinese character and display its associated phrases for selection
@@ -263,7 +277,7 @@ class StrokeOrder():
 		window.set_app_menu(menu)	
 		window.add(vbox)	
 		window.show_all()		
-		self.textArea.grab_focus()		
+		self.textArea.grab_focus()
 		gtk.main()			
 		
 	def aboutProduct(self, AboutProductButton, window):
@@ -274,7 +288,106 @@ class StrokeOrder():
 		response = dialog.run()
 		dialog.destroy()
 
+	def queryStrokesClicked(self, QueryStrokesButton, window):
+		# Create the UI for the stroke query module
+		
+		notebook = gtk.Notebook()
+		queryCharLabel = gtk.Label()
+		queryCharLabel.set_justify(gtk.JUSTIFY_LEFT)
+		queryCharLabel.set_width_chars(25)
+		queryCharLabel.set_line_wrap(True)		
+		queryCharLabel.set_text(_("StrokeOrder_QueryStrokesInstruction"))
+		
+		self.queryCharEntry = hildon.Entry(gtk.HILDON_SIZE_AUTO)				
+		self.queryCharEntry.set_max_length(1)
+		
+		queryCharEntryFont=pango.FontDescription('normal 60')
+		self.queryCharEntry.modify_font(queryCharEntryFont)
+		self.queryCharEntry.set_size_request(120,120)
+		
+		PasteButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+		PasteButton.set_label(_("StrokeOrder_Paste"))
+		PasteButton.connect("clicked", self.queryStrokesPasteClicked)
+		
+		queryCharHBox = gtk.HBox()
+		queryCharHBox.pack_start(queryCharLabel, True, True, 0)		
+		queryCharHBox.pack_start(self.queryCharEntry, False, True, 0)		
+		
+		queryCharVBox = gtk.VBox()
+		queryCharVBox.pack_start(queryCharHBox, False, True, 0)		
+		queryCharVBox.pack_start(PasteButton, True, True, 0)		
+			
+		self.queryStrokesResultLabel = gtk.Label()		
+		self.queryStrokesResultExplainLabel = gtk.Label()		
+
+		self.queryStrokesCopyButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+		self.queryStrokesCopyButton.set_label(_("StrokeOrder_Copy"))
+		self.queryStrokesCopyButton.connect("clicked", self.queryStrokesCopyClicked, window)
+		resultVBox = gtk.VBox()
+		resultVBox.pack_start(self.queryStrokesResultExplainLabel, True, True, 0)
+		
+		queryStrokesPannableArea = hildon.PannableArea()
+		queryStrokesPannableArea.set_property("mov-mode", hildon.MOVEMENT_MODE_BOTH)
+		queryStrokesPannableArea.add_with_viewport(self.queryStrokesResultLabel)
+		
+		resultVBox.pack_start(queryStrokesPannableArea, True, True, 0)
+		resultVBox.pack_start(self.queryStrokesCopyButton, True, True, 0)
+						
+		notebook.append_page(queryCharVBox, None)
+		notebook.append_page(resultVBox, None)
+		dialog = hildon.WizardDialog(None, _("StrokeOrder_QueryStrokes"), notebook)
+		notebook.connect("switch-page", self.queryStrokesStartQuery, dialog)
+		dialog.set_forward_page_func(self.queryStrokesWizardNextFunc)
+		dialog.show_all()
+		response = dialog.run()
+		if response == hildon.WIZARD_DIALOG_FINISH: dialog.destroy()
+
+	def queryStrokesWizardNextFunc(self, page, num, dialog):
+		# Define the criteria for proceeding to the next step
+		
+		if self.queryCharEntry.get_text() == "": 
+			return False
+		else:
+			return True
+
+	def queryStrokesCopyClicked(self, CopyButton, window):
+		# Copy the strokes to the clipboard
+		
+		textToBeCopied = self.queryStrokesResultLabel.get_text()
+		if textToBeCopied != _("StrokeOrder_QueryStrokesResultNotFound") and textToBeCopied != _("StrokeOrder_SymMode") and textToBeCopied != _("StrokeOrder_EngMode"):
+			clip = gtk.Clipboard()
+			clip.set_text( textToBeCopied )
+			clip.store()		
+
+	def queryStrokesPasteClicked(self, window):
+		# Triggered when user clicks the Paste button from the menu
+		clip = gtk.Clipboard()
+		data = clip.wait_for_contents('UTF8_STRING')
+		text = unicode( str(data.get_text()), "utf-8")
+		text = text[0:1]
+		self.queryCharEntry.set_text(text)
+
+			
+	def queryStrokesStartQuery(self, notebook, page, num, dialog):
+		# Performed the stroke query operation
+		
+		result = self.queryStrokes( unicode(self.queryCharEntry.get_text(),"utf-8") )
+		print result;
+		if result != "":
+			if result=="6" or result=="7":
+				self.queryStrokesCopyButton.set_sensitive(False)
+			else:
+				self.queryStrokesCopyButton.set_sensitive(True)
+			self.queryStrokesResultExplainLabel.set_text("「" + self.queryCharEntry.get_text() + _("StrokeOrder_QueryStrokesResultExplain"))				
+			self.queryStrokesResultLabel.set_text( self.ConvertCodeToChar(result) )
+		else:
+			self.queryStrokesResultLabel.set_text( _("StrokeOrder_QueryStrokesResultNotFound") )
+			self.queryStrokesCopyButton.set_sensitive(False)
+		return True
+	
 	def SMSPageClicked(self, window):
+		# Proceed to the SMS Preview page
+		
 		smsText = self.textAreaBuffer.get_text(self.textAreaBuffer.get_start_iter(), self.textAreaBuffer.get_end_iter(), False)
 		
 		if (smsText!=""):
@@ -410,7 +523,8 @@ class StrokeOrder():
 			
 			vbox.pack_start(sendToAndSelectContactButtonHBox, False, False, 0)						
 			vbox.pack_start(textAndnumpadVBox, True, True, 0)		
-			
+
+			sendSMSPage.connect("delete_event",self.sendSMSPageClosed)
 			sendSMSPage.add(vbox)
 			menu = self.create_menu(sendSMSPage, "SMSPage") 
 			sendSMSPage.set_app_menu(menu)				
@@ -418,6 +532,10 @@ class StrokeOrder():
 		else:					
 			banner = hildon.hildon_banner_show_information(window,"" , _("StrokeOrder_Error_BlankMessage"))
 			
+	def sendSMSPageClosed(self, widnow, widget):
+		# Set the input focus back to the text area once the SMS Preview page is closed
+		self.textArea.grab_focus()
+	
 	def rememberContactNumbers(self, widget):
 		# For remembering the contact numbers when user switches back and forth the pages
 		self.sendToNumbers = self.sendToBuffer.get_text(self.sendToBuffer.get_start_iter(), self.sendToBuffer.get_end_iter())	# For remembering the contact numbers when user switches back and forth the pages
@@ -447,16 +565,20 @@ class StrokeOrder():
 		NewButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
 		NewButton.set_label(_("StrokeOrder_New"))   
 		NewButton.connect("clicked", self.NewClicked)			
+
+		QueryStrokesButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+		QueryStrokesButton.set_label(_("StrokeOrder_QueryStrokes"))   
+		QueryStrokesButton.connect("clicked", self.queryStrokesClicked, window)			
+		
 		if page == "MainPage":
 			NewButton.set_sensitive(True)
 		elif page == "SMSPage":
 			NewButton.set_sensitive(False)	# Disable button if user is using SMS page
-		menu.append(NewButton)
-			
-		menu.append(AboutButton)   		
-		
+		menu.append(NewButton)							
+		menu.append(QueryStrokesButton)
 		menu.append(CopyButton)
 		menu.append(PasteButton)		
+		menu.append(AboutButton)   		
 
 		menu.show_all()
 		return menu
@@ -486,10 +608,13 @@ class StrokeOrder():
 		
 		if stroke==6 or stroke==7:	# Do not build up stroke sequence if English or Symbol button is pressed
 			self.strokesCodeInputted = str(stroke)				
-		elif "6" in self.strokesCodeInputted or "7" in self.strokesCodeInputted:
+		elif ("6" in self.strokesCodeInputted or "7" in self.strokesCodeInputted) and stroke!=8:
 			self.strokesCodeInputted = str(stroke)
 		elif stroke==8:
-			self.strokesCodeInputted = self.strokesCodeInputted + "%"
+			if ("6" in self.strokesCodeInputted or "7" in self.strokesCodeInputted):
+				self.strokesCodeInputted = "%"
+			else:
+				self.strokesCodeInputted = self.strokesCodeInputted + "%"
 		else:
 			self.strokesCodeInputted = self.strokesCodeInputted + str(stroke)
 		self.strokesInputted.set_text( self.ConvertCodeToChar(self.strokesCodeInputted) )	# Display the inputted strokes
@@ -509,12 +634,14 @@ class StrokeOrder():
 		selectedText = buffer.get_selection_bounds()
 		if selectedText!=(): # If there is selected text, delete the selected text
 			buffer.delete_selection(True,True)
-		buffer.set_text(buffer.get_text(buffer.get_start_iter(), buffer.get_iter_at_mark(buffer.get_insert()), False) + text + buffer.get_text(buffer.get_iter_at_mark(buffer.get_insert()), buffer.get_end_iter(), False))
+		buffer.insert(buffer.get_iter_at_mark(buffer.get_insert()), text)		
  
 		if self.textboxInFocus == "textArea":
 			self.textArea.set_buffer(buffer)
+			self.textArea.grab_focus()
 		elif self.textboxInFocus == "sendTo":
 			self.sendTo.set_buffer(buffer)
+			self.sendTo.grab_focus()
 	
 	def CharSelected(self, Selector, column):
 		# Triggered once a candidate / English / Numeric / Symbol character is selected from the TouchSelector control.
@@ -524,8 +651,9 @@ class StrokeOrder():
 		
 #		if self.textboxInFocus == "textArea":	# If text area is having input focus, insert text to the current cursor position
 		buffer = self.textArea.get_buffer()
-		buffer.set_text(buffer.get_text(buffer.get_start_iter(), buffer.get_iter_at_mark(buffer.get_insert()), False) + text + buffer.get_text(buffer.get_iter_at_mark(buffer.get_insert()), buffer.get_end_iter(), False))			
-		self.textArea.set_buffer(buffer)	
+		buffer.insert(buffer.get_iter_at_mark(buffer.get_insert()), text)
+		self.textArea.grab_focus()
+		
 #		elif self.textboxInFocus == "sendTo":	# If phone number field is having input focus, insert text to the current cursor position
 #			buffer = self.sendTo.get_buffer() 
 #			buffer.set_text(buffer.get_text(buffer.get_start_iter(), buffer.get_iter_at_mark(buffer.get_insert()), False) + text + buffer.get_text(buffer.get_iter_at_mark(buffer.get_insert()), buffer.get_end_iter(), False))
